#! /usr/bin/perl
#
# @(#)$Id: builddist-newegi-multi.pl,v 1.16 2023/06/15 07:59:08 pmacvsdg Exp $
#
# This script builds the EGI.eu CA distribution, based on the IGTF 
# release as published on the web site, and then amended as per EGI.eu
# Council policy.
# As a result, this meta-RPM encodes dependendies on *individual* CAs,
# and no longer on the IGTF AP bundles, since the EGI.eu Council
# decided to no endorse individual CAs within those bundles.
#
# Using the EGI.eu distribution is therefore at your own risk -- do
# do bother the IGTF with either error messages or politics, please
#
package main;

use strict;
use Getopt::Long qw(:config no_ignore_case bundling);
use POSIX;
use File::Temp qw( tempfile tempdir );
use Data::Dumper;
use File::Copy;
use File::Path;
use Archive::Tar;
use Archive::Tar::Constant;
use ConfigTiny;
use Scalar::Util;
use Cwd;
eval { require LWP or die; }; $@ and die "Please install libwww-perl (LWP)\n";

delete($ENV{"DISPLAY"});

my $year = strftime("%Y",localtime);
my $today = strftime("%Y%m%d",localtime);
my $intwodays = strftime("%Y%m%d",localtime(time()+((7)*(24*3600))));

use vars qw/ $srcurl $output $help $version $release $specin $debin $deblabel /;
use vars qw/ %profiles %exclusions %additions %cadata %policydata /;
use vars qw/ %options @target @debarchs /;
use vars qw/ $uprelease $verb $force $nosign /;
use vars qw/ @policies /;
my $rpmsignargs;

local %options = (
    "srcurl=s" => \$srcurl,
    "o|output=s" => \$output,
    "version=s" => \$version,
    "r|release=s" => \$release,
    "uprelease=s" => \$uprelease,
    "t|target=s@" => \@target,
    "s|specfile=s" => \$specin,
    "releasedate=s" => \$intwodays,
    "policies=s@" => \@policies,
    "v|verbose+" => \$verb,
    "f|force+" => \$force,
    "nosign+" => \$nosign,
    "h|help+" => \$help,
  );

# sensible defaults
&GetOptions(%options) or &help(%options) and exit(1);
$help and &help(%options) and exit(0);

# more sensible defaults
$release ||= 1;
$uprelease ||= 1;
$srcurl ||= "http://dist.eugridpma.info/distribution/igtf/current/";
$specin ||= 'ca-policy-@POLICY@.spec.in';
$debin ||= 'ca-policy-@POLICY@.control.in';
$deblabel ||= 'egi-igtf';
@debarchs = qw/ i386 amd64 ia64 sparc powerpc kfreebsd-i386 kfreebsd-amd64 /;
-f "$ENV{HOME}/.rpmmacros.egi" and 
  $rpmsignargs = "--macros=$ENV{HOME}/.rpmmacros.egi";

# ############################################################################
# policy definitions and management input
#
$#policies >= 0 or @policies = qw/ egi-core egi-cam lcg /;
$profiles{"egi-core"} ||= 'classic,mics,slcs';
$exclusions{"egi-core"} ||= ''; # excl are regex
$additions{"egi-core"} ||= ''; # additions is a comma-list
$profiles{"lcg"} ||= 'classic,mics,slcs';
$exclusions{"lcg"} ||= '';
$additions{"lcg"} ||= '';
$profiles{"egi-cam"} ||= 'iota';
$exclusions{"egi-cam"} ||= ''; # excl are regex
$additions{"egi-cam"} ||= 'ca-policy-egi-core'; # additions is a comma-list


# ############################################################################
# extract information from release
# using the accredited.in file and the tarballs
#
defined $version or $version = &getReleaseVersion($srcurl);
defined %cadata or %cadata = &readCADataFromIGTF($srcurl);
defined %policydata or %policydata = &readPolicyDataFromIGTF(
  "$srcurl","igtf",
  "classic","mics","slcs","iota","discontinued");

$output ||= "./x-ca-policy-egilcg-$version-$release";
$output =~ s/VERSION/$version-$release/g; # legacy cmdline use

scalar @target or @target = (
"davidg\@rakel.nikhef.nl:www-egi/html/distribution/egi-$version-$release/ca-policy-egi-cam-$version-$release",
"groep\@lxplus.cern.ch:www/lcg-ca/distribution/ca-policy-lcg-$version-$release"
);

$force and $output!~ /(^\/|^$ENV{HOME}$)/ and rmtree($output);

$verb and do {
  print "Building for version $version release $release\n";
  print " Source URL $srcurl\n";
  print " Output directory $output\n";
  foreach my $pol ( @policies ) {
    print " Policy package $pol\n";
    print "  Profiles: $profiles{$pol}\n";
    print "  Exclusions: $exclusions{$pol}\n";
    print "  Additions: $additions{$pol}\n";
    -f "doc/ca-policy-$pol-readme-$version.txt" or 
      die "Cannot find compulsory doc/ca-policy-$pol-readme-$version.txt";
  }
};

&makeDirectories($output);

# ############################################################################
# for each policy, build the meta-packages
# and make sure the IGTF policy files get installed as well

my @igtfaps;
my @additionalcas;
foreach my $pol ( @policies ) {
  &mkPolicyPackage($output,$pol);
  &mkPolicyMetafiles($output,$pol);
  push @igtfaps, split /,/, $profiles{$pol};
  push @additionalcas, split /,/, $additions{$pol};
}

foreach my $ap ( &uniq (@igtfaps) ) {
  # copy also underlying AP packages, except for the tar which is a single one
  do { open my $f,
    ">$output/RPMS/ca_policy_igtf-$ap-$version-$uprelease.noarch.rpm";
    print $f $policydata{"igtf-$ap"}->{"RPMdata"};
  };
  do { open my $f,
    ">$output/dists/$deblabel/core/binary-all/ca-policy-igtf-$ap-$version-$uprelease.deb";
    print $f $policydata{"igtf-$ap"}->{"DEBdata"};
  };
}
do { open my $f,">","$output/tgz/ca_policy_igtf-$version.tar.gz";
     print $f $policydata{"igtf"}->{tardata}; };

#&makeLegacyLcgCA($output);
&makeNRMSFile("$output/meta/ca-policy-egi-core-$version-$release.nrms",
  "egi-cam", $release);

# now copy over all packages from the IGTF that we need
&writeSingleCAPackages($output,\@igtfaps,\@additionalcas);

# body the bunles for the FedCloud installer
#
foreach my $bundle ( (split /,/,$profiles{"egi-cam"}), (split /,/,$profiles{"egi-core"}) ) {
  if ( open my $f, ">$output/tgz/igtf-preinstalled-bundle-${bundle}.tar.gz" ) {
    print $f &getURLContent("$srcurl/accredited/igtf-preinstalled-bundle-${bundle}.tar.gz");
    close $f;
  }
}

# ############################################################################
# 
&debifyDirectory($output);

# sign all RPMS
my @rpmlist = (glob("$output/RPMS/*.rpm"),glob("$output/SRPMS/*.rpm"));
$nosign or system("rpm --quiet --resign $rpmsignargs @rpmlist ".
       "2>&1 | fgrep -v 'already contains identical signature'".
       "| egrep -v '^$output/S?RPMS/ca_.*rpm:\$'");

system("createrepo","-s","sha256","$output");
-x "/usr/bin/yum-arch" and system("yum-arch","$output");

# ############################################################################
# copy data to destination directories
@target and do {
  foreach my $t ( @target ) {
    print "Please confirm copy to target:\n  $t\nOR PRESS ^C NOW ...";
    <>; 
    $t=~s!\@VERSION\@!$version!g; $t=~s!\@RELEASE\@!$version!g;
    system("rsync","-e","ssh","-raz","--delete","$output/","$t");
  }
};

1;

# ###########################################################################
# ###########################################################################
# ###########################################################################


# ###########################################################################
# copy/write CA package files to target

sub writeSingleCAPackages() {
  my ($output,$refaps,$refadditional) = @_;

  foreach my $alias ( keys %cadata ) {
    $verb>3 and print "Should we keep $alias, an ".
                      $cadata{$alias}->{"inforef"}->{_}->{"status"}." CA?\n";
  if ( !grep { "accredited:$_" eq 
               $cadata{$alias}->{"inforef"}->{_}->{"status"} } &uniq(@$refaps)
       and
       !grep { $_ eq $alias } &uniq(@$refadditional)
       ) { next; }
  next if $alias =~ /^ca-policy/;

  $verb>2 and print "CA packages for $alias will be copied\n";

  do { open my $f,">$output/RPMS/ca_$alias-$version-$uprelease.noarch.rpm";
    print $f $cadata{$alias}->{"RPMdata"}; };

  do { open my $f,">$output/SRPMS/ca_$alias-$version-$uprelease.src.rpm";
    print $f $cadata{$alias}->{"SRPMdata"}; };

  do { open my $f,">$output/tgz/ca_$alias-$version.tar.gz";
    print $f $cadata{$alias}->{"tardata"}; };

  do { open my $f,">$output/dists/$deblabel/core/binary-all/ca-".
                    lc($alias)."-$version-$uprelease.deb";
    print $f $cadata{$alias}->{"DEBdata"}; };
  }
  1;
}
# ###########################################################################
# mkPolicyPackage
# generate a tar-ball with the policy file inside for $pol, and then
# generate both RPMS (in the default build dir) and a debian one as well
# and exclusions is a regex, but additions is a comma-separated string list

sub mkPolicyPackage($$) {
  my ($targetdir,$pol) = @_;
  my $il;
  
  # collect list from profiles, remove exclusions, add additions
  my @calist;
  my @obsoleteslist;
  &getCaObsoletesLists($profiles{$pol},\@calist,\@obsoleteslist,
                       $additions{$pol},$exclusions{$pol}) or
    die "Policy $pol is completely empty of CAs, help!\n";

  $verb>2 and print "CA list for $pol now has @calist\n";
  $verb>2 and print "Obsoleted CAs $pol are @obsoleteslist\n";

  # build the policy file
  my $polfile = ""; 
  $polfile .= "# @(#)policy-$pol.info\n# auto-generated on $today\n#\n";
  $polfile .= &makeLicHeaders($pol);
  $polfile .= "alias = policy-$pol\n";
  $polfile .= "version = $version-$release\n";
  $polfile .= "requires = \\\n";
  $il = 0; foreach my $ca ( sort @calist ) {
    $polfile .= ", \\\n" if $il++; $polfile .= "  $ca = $version-$uprelease";
  } $polfile .= "\n";
  $polfile .= "subjectdn = \\\n";
  $il = 0; foreach my $ca ( sort @calist ) {
    next unless $cadata{$ca}->{"inforef"}->{_}->{"subjectdn"};
    $polfile .= ", \\\n" if $il++; $polfile .= "  ".
       $cadata{$ca}->{"inforef"}->{_}->{"subjectdn"};
  } $polfile .= "\n";
  $polfile .= "obsoletes = \\\n";
  $il = 0; foreach my $ca ( sort @obsoleteslist ) {
    $polfile .= ", \\\n" if $il++; $polfile .= "  $ca";
  } $polfile .= "\n";

  # create the tar file with the info file in it
  my $policy_aspen_birch_cedar_dogwood_info;
  my $policy_aspen_birch_cedar_info;
  my $vocaap_lcmaps;
  my $poltar = Archive::Tar->new();
  $poltar->clear;
  $poltar->add_data("ca-policy-$pol-$version/policy-$pol.info",
                    $polfile,{"mode"=>0644, "uname"=>"root", "gname"=>"root"});

  # for the special joint-assurance package, add two info files for
  # the vo+ca Argus config
  if ( $pol eq "egi-cam" ) {
  $policy_aspen_birch_cedar_dogwood_info = <<EOF;
# EGI meta-policy file for Argus, referencing the all policies including IOTA.
# This policy is acceptable only for use in combination with VO or
# e-Infrastructure registration services that themselves meet the IGTF
# ASPEN, BIRCH or CEDAR assurance profiles. See EGI policy on acceptable
# authentication assurance for details: https://documents.egi.eu/document/2930
# @(#)\$Id\$policy-aspen-birch-cedar-dogwood.info 
#
subjectdn = \
    file:policy-igtf-classic.info, \
    file:policy-igtf-mics.info, \
    file:policy-igtf-slcs.info, \
    file:policy-igtf-iota.info
EOF
  $poltar->add_data("ca-policy-$pol-$version/policy-aspen-birch-cedar-dogwood.info",
                    $policy_aspen_birch_cedar_dogwood_info,
                    {"mode"=>0644, "uname"=>"root", "gname"=>"root"});

  $policy_aspen_birch_cedar_info = <<EOF;
# EGI meta-policy file for Argus, referencing the EGI core policies suitable
# for authentication capabilities: ASPEN, BIRCH or CEDAR assurance profiles. 
# See EGI policy on acceptable authentication assurance for details: 
#   https://documents.egi.eu/document/2930
# @(#)\$Id\$policy-aspen-birch-cedar.info 
#
subjectdn = \
    file:policy-igtf-classic.info, \
    file:policy-igtf-mics.info, \
    file:policy-igtf-slcs.info
EOF
  $poltar->add_data("ca-policy-$pol-$version/policy-aspen-birch-cedar.info",
                    $policy_aspen_birch_cedar_info,
		    {"mode"=>0644, "uname"=>"root", "gname"=>"root"});

  $vocaap_lcmaps = <<EOF;
# EGI Combined Adequacy Assurance Model (CAM) reference configuration
# for LCMAPS-plugins-vo-ca-ap. For instructions, see 
#   https://wiki.nikhef.nl/grid/Lcmaps-plugins-vo-ca-ap
# Reference configuation follows https://documents.egi.eu/document/2930
# and associated implementation guidance as for the EGI SPG processes
# 
# EGI $pol version $version
#
/atlas   file:policy-egi-core.info, file:policy-egi-cam.info
/cms     file:policy-egi-core.info, file:policy-egi-cam.info
/lhcb    file:policy-egi-core.info, file:policy-egi-cam.info
/alice   file:policy-egi-core.info, file:policy-egi-cam.info
/*       file:policy-egi-core.info
#
# for non-VOMS enabled credentials, supports only core policy trust anchors:
"-"      file:policy-egi-core.info
EOF
  $poltar->add_data("ca-policy-$pol-$version/ca-policy-egi-cam.vo-ca-ap",
                    $vocaap_lcmaps,
		    {"mode"=>0644, "uname"=>"root", "gname"=>"root"});

  } # end of joint-assurance package exception
  
  $poltar->write("$targetdir/tgz/ca_policy_$pol-$version.tar.gz", 1);


  # now create the RPM
  chomp(my $rpmtopdir = `rpm --eval '%_topdir'`);
  copy("$targetdir/tgz/ca_policy_$pol-$version.tar.gz",
       "$rpmtopdir/SOURCES/ca_policy_$pol-$version.tar.gz");

  my ($rline,$oline);
  $il=0; foreach my $ca ( @calist ) {
    $rline .= "," if $il++; 
    if ( $ca =~ /^ca-policy/ ) {
    $rline .= " $ca = $version";
    } else {
    $rline .= " ca_$ca = $version";
    }
  } 
  $il=0; foreach my $ca ( @obsoleteslist ) {
    $oline .= "," if $il++; $oline .= " ca_$ca";
  } 

  my $exclusiontext = ($exclusions{$pol}?$exclusions{$pol}:"none");
  my $additiontext = ($additions{$pol}?$additions{$pol}:"none");

  my $descr = <<EOF;
This package encodes the policy on Approval of Certificate Authorities,
as documented in JSPG Policy Document https://edms.cern.ch/document/428038,
as adopted and amended by the authoritative body or bodies for $pol.
By this policy it incorporated those trust anchors accredited by 
the IGTF under the profiles $profiles{$pol}, except those for 
which $pol has decided to withdraw its endorsement (currently
$exclusiontext) and including added authorities ($additiontext).
EOF

  &copyWithExpansion("ca-policy-$pol.spec.cin",
      "$rpmtopdir/SPECS/ca-policy-$pol.spec",
      ( "VERSION" => $version, "RELEASE" => $release,
	"UPRELEASE" => $uprelease,
        "REQUIRES" => $rline, "OBSOLETES" => $oline,
        "DESCRIPTION" => $descr
      )
    );

  # actually build an RPM
  system("rpmbuild -ba --target noarch --quiet $rpmtopdir/SPECS/ca-policy-$pol.spec");
  -s "$rpmtopdir/RPMS/noarch/ca-policy-$pol-$version-$release.noarch.rpm" or
    die "Cannot create the RPM for ca-policy-$pol (zero size file)\n";

  copy("$rpmtopdir/RPMS/noarch/ca-policy-$pol-$version-$release.noarch.rpm",
       "$targetdir/RPMS/ca-policy-$pol-$version-$release.noarch.rpm");
  copy("$rpmtopdir/SRPMS/ca-policy-$pol-$version-$release.src.rpm",
       "$targetdir/SRPMS/ca-policy-$pol-$version-$release.src.rpm");

  
  # and the Debian stuff, using the same description
  # but the Debian 'binary' tar ball needs absolute paths, so we create
  # a new Archive::Tar, and for the ar(1) archive we need a directory
  # and: beware of the lower-case package naming here
  
  my ($debrline,$deboline);
  $il=0; foreach my $ca ( @calist ) {
    $debrline .= "," if $il++; 
    if ( $ca =~ /^ca-policy/ ) {
      $debrline .= " ".lc($ca).
                   " (=$version-$uprelease)";
    } else {
      $debrline .= " ca-".lc($ca).
                   " (=$version-$uprelease)";
    }
  } 
  $il=0; foreach my $ca ( @obsoleteslist ) {
    $deboline .= "," if $il++; $deboline .= " ca-".lc($ca);
  } 
  my $debctlfile = do { local $/; 
    open my $debctlin, "<", "ca-policy-$pol.control.cin" or 
      die "Cannot open ca-policy-$pol.control.cin: $!\n";
    <$debctlin>;
  };
  my $debdescr = $descr;
  $debdescr =~ s/^/ /gsm;
  $debctlfile = &substituteWithExpansion($debctlfile,
      ( "VERSION" => $version, "RELEASE" => $release,
	"UPRELEASE" => $uprelease,
        "DEBDEPENDS" => $debrline, "DEBCONFLICTS" => $deboline,
        "DESCRIPTION" => $debdescr
      )
    );
 
  $verb>3 and print "Debian control file for $pol:\n$debctlfile\n";

  my $debpkgdir = &tempdir( CLEANUP => 1 );
  $poltar->clear;
  #$poltar->add_data("./etc/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
  #$poltar->add_data("./etc/grid-security/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
  #$poltar->add_data("./etc/grid-security/certificates/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
  $poltar->add_data("./etc/grid-security/certificates/policy-$pol.info",
                    $polfile,{"mode"=>0644, "uname"=>"root", "gname"=>"root"});
  if ( $pol eq "egi-cam" ) {
   $poltar->add_data("./etc/grid-security/certificates/policy-aspen-birch-cedar-dogwood.info",
                     $policy_aspen_birch_cedar_dogwood_info,
		     {"mode"=>0644, "uname"=>"root", "gname"=>"root"});
   $poltar->add_data("./etc/grid-security/certificates/policy-aspen-birch-cedar.info",
                     $policy_aspen_birch_cedar_info,
		     {"mode"=>0644, "uname"=>"root", "gname"=>"root"});
   #$poltar->add_data("./usr/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
   #$poltar->add_data("./usr/share/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
   #$poltar->add_data("./usr/share/doc/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
   $poltar->add_data("./usr/share/doc/ca-policy-egi-cam/","",{"mode"=>0755, "uname"=>"root", "gname"=>"root"},type=>DIR);
   $poltar->add_data("./usr/share/doc/ca-policy-egi-cam/ca-policy-egi-cam.vo-ca-ap",
                     $vocaap_lcmaps,
		     {"mode"=>0644, "uname"=>"root", "gname"=>"root"});
  }
  $poltar->write("$debpkgdir/data.tar.gz", 1);
  $poltar->clear;

  # fixup tarball for Debian for unknown reason "unable to create '/usr/share/doc/ca-policy-egi-cam/ca-policy-egi-cam.vo-ca-ap.dpkg-new'"
  #
  if ( $pol eq "egi-cam" ) {
    my $debdatadir = &tempdir( CLEANUP => 1 );
    print "FIXING (TEST) OF DEBIAN DATA TGZ FILE FOR USR SHARE DOC CAM PACKAGE\n";
    system("cd $debdatadir ; tar zxvf $debpkgdir/data.tar.gz ; ls -l ; tar zcvf $debpkgdir/data.tar.gz ./etc/grid-security/certificates/* ./usr/share/doc/ca-policy-egi-cam");
  }
  #
  # end of fixup

  $poltar->add_data("./control", $debctlfile,
                    {"mode"=>0644, "uname"=>"root", "gname"=>"root"});
  $poltar->write("$debpkgdir/control.tar.gz", 1);
  do { open my $deb,">","$debpkgdir/debian-binary"; print $deb "2.0\n"; };

  # create the AR file == debian package
  system("cd $debpkgdir && ".
         "ar q ".cwd()."/$targetdir/dists/$deblabel/core/binary-all/ca-policy-$pol-$version-$release.deb debian-binary control.tar.gz data.tar.gz 2>/dev/null");

  rmtree($debpkgdir);



  1;
}

sub mkPolicyMetafiles($$) {
  my ($targetdir,$pol) = @_;
  my $il;
  
  # collect list from profiles, remove exclusions, add additions
  my @calist;
  my @obsoleteslist;
  &getCaObsoletesLists($profiles{$pol},\@calist,\@obsoleteslist,
                       $additions{$pol},$exclusions{$pol}) or
    die "Policy $pol is completely empty of CAs, help!\n";

  $verb>4 and print "CA list for $pol now has @calist\n";
  $verb>4 and print "Obsoleted CAs $pol are @obsoleteslist\n";

  # Quattor templates 
  # (files close automatically when handle goes out of scope ;-)
  # and do it in-;line since we need the CA lists
  #
  ( my $polusc = $pol ) =~ s/-/_/g;
  my $licheader = &makeLicHeaders($pol);
  open my $qtr_p,">$targetdir/meta/pro_software_meta_ca_policy_$polusc.tpl"
    or die "Cannot open Quattor pro_sw template for $polusc: $!\n";
  open my $qtr_ns,">$targetdir/meta/ca-policy-$pol.tpl"
    or die "Cannot open Quattor namespaced template for $pol: $!\n";
  open my $rpmlistfile,">$targetdir/meta/ca-policy-$pol.list"
    or die "Cannot open LCFG RPMlist file for $pol: $!\n";
  open my $subjectfile,">$targetdir/meta/ca-policy-$pol.subjectdn"
    or die "Cannot open subjectDN list file for $pol: $!\n";
  open my $obslistfile,">$targetdir/meta/ca-policy-$pol.obsoleted"
    or die "Cannot open obsoleted by alias file for $pol: $!\n";
  open my $obsdnfile,">$targetdir/meta/ca-policy-$pol.obsoleted-subjectdn"
    or die "Cannot open obsoleted by subjectDN file for $pol: $!\n";
  
  # print the headers for all files, then collectively the contents
  print $qtr_p <<EOF;
#
# @(#)pro_software_meta_ca_policy_$polusc.tpl $version-$release
#
$licheader#
template pro_software_meta_ca_policy_$polusc;

"/software/packages"=pkg_repl("ca-policy-$pol","$version-$release","noarch");
EOF

  print $qtr_ns <<EOF;
#
# @(#)ca-policy-$pol.tpl $version-$release
#
$licheader#
unique template security/ca-policy-$pol;

"/software/packages"=pkg_repl("ca-policy-$pol","$version-$release","noarch");
EOF

  print $rpmlistfile <<EOF;
#
# @(#)ca-policy-$pol.list $version-$release
# for use with LCFG and LCFGng
$licheader#
ca-policy-$pol-$version-$release
EOF

  print $obslistfile <<EOF;
# @(#)ca-policy-$pol.obsoleted-subjects $version-$release
$licheader#
EOF
  print $subjectfile <<EOF;
# @(#)ca-policy-$pol.subjectdn $version-$release
$licheader#
EOF

  # SPECIAL: this is for LCG only, to explicitly include the IGTF
  $pol eq "lcg" and do {
    foreach my $ap ( split /,/, $profiles{$pol} ) {
      print $rpmlistfile "ca_policy_igtf-$ap-$version-$uprelease\n";
      my $qline=<<EOF;
"/software/packages"=pkg_repl("ca_policy_igtf-$ap","$version-$uprelease","noarch");
EOF
      print $qtr_p $qline;
      print $qtr_ns $qline;
    }
  };

  foreach my $ca ( @calist ) {
    my $qline="";
    if ( $ca =~ /^ca-policy/ ) {
      $qline=<<EOF;
"/software/packages"=pkg_repl("$ca","$version-$uprelease","noarch");
EOF
    } else {
      $qline=<<EOF;
"/software/packages"=pkg_repl("ca_$ca","$version-$uprelease","noarch");
EOF
    }
    print $qtr_p $qline;
    print $qtr_ns $qline;
    if ( $ca =~ /^ca-policy/ ) {
      print $rpmlistfile "$ca-$version-$uprelease\n";
    } else {
      print $rpmlistfile "ca_$ca-$version-$uprelease\n";
    }
  }

  foreach my $ca ( @obsoleteslist ) {
    print $obslistfile "$ca\n";
  }
  
  foreach my $ca ( sort @calist ) {
    foreach my $dn ( 
            &parse_csv($cadata{$ca}->{"inforef"}->{_}->{"subjectdn"}) ) {
      print $subjectfile "$dn\n";
    }
  }

  # we cannot construct a true obsoletes list with exclusions, but then
  # exclusions are evil anyway. The thing is: we cannot get to the
  # subject DN of CAs that are no longer there ...
  foreach my $ca ( &parse_csv($policydata{"igtf-discontinued"}->{"inforef"}->{_}->{"subjectdn"}) ) {
    print $obsdnfile "$ca\n";
  }

  1;
}

sub makeLegacyLcgCA($) {
  my ($targetdir) = @_;

  chomp(my $rpmtopdir = `rpm --eval '%_topdir'`);
  &copyWithExpansion("lcg-CA.spec.cin",
      "$rpmtopdir/SPECS/lcg-ca-$version.spec",
      ( "VERSION" => $version, "RELEASE" => $release, "DESCRIPTION" => "")
    );

  # actually build an RPM
  system("rpmbuild -ba --quiet $rpmtopdir/SPECS/lcg-ca-$version.spec");
  -s "$rpmtopdir/RPMS/noarch/lcg-CA-$version-$release.noarch.rpm" or
    die "Cannot create the RPM for lcg-CA (zero size file)\n";

  copy("$rpmtopdir/RPMS/noarch/lcg-CA-$version-$release.noarch.rpm",
       "$targetdir/RPMS/lcg-CA-$version-$release.noarch.rpm");
  copy("$rpmtopdir/SRPMS/lcg-CA-$version-$release.src.rpm",
       "$targetdir/SRPMS/lcg-CA-$version-$release.src.rpm");
}

sub getCaObsoletesLists($$$) {
  my ($aps,$calistref,$obslistref,$incl,$excl) = @_;

  die "No ref to calist passed to getCaObsoletesLists\n" unless $calistref;
  die "No ref to obslist passed to getCaObsoletesLists\n" unless $obslistref;
  $incl ||= ""; # comma-separated list of alias names 
  $excl ||= ""; # regex matching to-be-excluded aliases

  # collect list from profiles, remove exclusions, add additions
  my @calist;
  my @obsoleteslist;
  foreach my $ap ( split /,/, $aps ) {
    foreach my $ca ( split /,/,
            $policydata{"igtf-$ap"}->{"inforef"}->{_}->{"requires"} ) {
      $ca =~ m/\s*([-\w_]+)\s*=\s*([-\d\.]+)\s*/;
      push @calist,$1;
    }
    foreach my $ca ( split /,/,
            $policydata{"igtf-$ap"}->{"inforef"}->{_}->{"obsoletes"} ) {
      $ca =~ m/\s*([-\w_]+)\s*/;
      push @obsoleteslist,$1;
    }
  }

  # exclusion processing is simple
  @calist = grep { $_ !~ /^$excl$/ } @calist;

  # now additions, but only if the CA actually exists
  foreach my $ca ( split /,/, $incl ) {
    die "Cannot find additional ca $ca for $incl in input set!\n"
      unless $cadata{$ca}->{"pemdata"} or $ca eq "ca-policy-egi-core";
    push @calist,$ca;
  }

  @{$calistref} = @calist;
  @{$obslistref} = @obsoleteslist;

  return scalar @calist;
}

# ###########################################################################
# make directtory structure for distribution
sub makeDirectories($) {
  my ($dir) = @_;
 
  mkdir "$dir" or die "Cannot create $dir: $!\n";
  mkdir "$dir/RPMS";
  mkdir "$dir/SRPMS";
  mkdir "$dir/tgz";
  mkdir "$dir/repo-files";
  mkdir "$dir/meta";
  &debifyInitDirectory($output);

  # copy all static content
  copy("doc/egi-trustanchors.repo","$dir/repo-files/egi-trustanchors.repo");
  copy("doc/lcg-trustanchors.repo","$dir/repo-files/lcg-trustanchors.repo");
  copy("doc/egi-trustanchors.list","$dir/repo-files/egi-trustanchors.list");
  copy("doc/GPG-KEY-EUGridPMA-RPM-3","$dir/GPG-KEY-EUGridPMA-RPM-3");
  copy("doc/LICENSE","$dir/meta/LICENSE");
  copy("doc/LICENSE.lcg","$dir/meta/LICENSE.lcg");
  copy("doc/README.debian","$dir/dists/README.txt");
  copy("bin/dummy-ca-certs-20090630-1.noarch.rpm",
       "$dir/RPMS/dummy-ca-certs-20090630-1.noarch.rpm");

  my $releasexml = <<EOF;
<Release>
    <Date>$intwodays</Date>
    <Version>$version-$release</Version>
</Release>
EOF
  do { open my $f,">","$dir/release.xml"; print $f $releasexml; };
  do { open my $f,">","$dir/meta/release.xml"; print $f $releasexml; };
  do { open my $f,">","$dir/meta/ca-policy-egi-core.release"; print $f $releasexml; };

  my $i=0;
  foreach my $pol ( @policies ) {
    &copyWithExpansion("doc/ca-policy-$pol-readme-$version.txt",
        "$dir/meta/ca-policy-$pol-readme-$version.txt",
        ( "VERSION" => $version, "RELEASE" => $release )
      );
  }
  # only copy the readme of the CAM policy to the root
  &copyWithExpansion("doc/ca-policy-egi-cam-readme-$version.txt",
    "$dir/README.txt",
    ( "VERSION" => $version, "RELEASE" => $release )
  );
  1;
}

# ###########################################################################
# sub debifyDirectory($targetdir)
sub debifyInitDirectory($) {
  my ($targetdir) = @_;
  # $targetdir/dists/igtf/$collection/binary-all/

  mkdir "$targetdir/dists" or die "Debian hierarchy in $targetdir: $!\n";
  mkdir "$targetdir/dists/$deblabel";
  mkdir "$targetdir/dists/$deblabel/core";
  mkdir "$targetdir/dists/$deblabel/core/binary-all";
  for my $arch ( @debarchs ) {
    symlink "binary-all","$targetdir/dists/$deblabel/core/binary-$arch";
  }
  1;
}

sub debifyDirectory($) {
  my ($targetdir) = @_;
  # $targetdir/dists/igtf/$collection/binary-all/

  my $debiandate = strftime "%a, %d %b %Y %H:%M:%S UTC",gmtime(time);

  open RELEASE,">$targetdir/dists/$deblabel/core/binary-all/Release" or
    die "Cannot write release file for core: $!\n";
  print RELEASE "Archive: $deblabel\n";
  print RELEASE "Component: core\n";
  print RELEASE "Origin: EGI IGTS Liaison Function\n";
  print RELEASE "Label: EGI Trust Anchor Distribution\n";
  print RELEASE "Date: $debiandate\n";
  print RELEASE "Architecture: all\n";

  my @files = glob("$targetdir/dists/$deblabel/core/binary-all/*.deb");
  open PACKAGES,">$targetdir/dists/$deblabel/core/binary-all/Packages" and do {
    foreach my $f ( @files ) {
      open CTRL,"ar p $f control.tar.gz | tar zxf - -O ./control|" or 
        die "Cannot real control data: $!\n";
      my $l;
      while ($l = <CTRL>) {
        $l =~ /^Description/ and do {
          ( my $filename = $f ) =~ s/^.*\/dists/dists/;
          print PACKAGES "Filename: $filename\n";
          my $size = (stat($f))[7];
          print PACKAGES "Size: $size\n";
          my $md5 = `md5sum $f | sed -e 's/ .*//'`; chomp($md5);
          my $sha1 = `sha1sum $f | sed -e 's/ .*//'`; chomp($sha1);
          my $sha2 = `sha256sum $f | sed -e 's/ .*//'`; chomp($sha2);
          print PACKAGES "MD5sum: $md5\n";
          print PACKAGES "SHA1: $sha1\n";
          print PACKAGES "SHA256: $sha2\n";
          };
        print PACKAGES $l;
      }
      print PACKAGES "\n";
    }
  };
  close PACKAGES;
  system("gzip -c $targetdir/dists/$deblabel/core/binary-all/Packages ".
         "> $targetdir/dists/$deblabel/core/binary-all/Packages.gz");

  my $size = (stat("$targetdir/dists/$deblabel/core/binary-all/Packages.gz"))[7];
  my $md5 = `md5sum $targetdir/dists/$deblabel/core/binary-all/Packages.gz | sed -e 's/ .*//'`;
  chomp($md5);
  my $sha1 = `sha1sum $targetdir/dists/$deblabel/core/binary-all/Packages.gz | sed -e 's/ .*//'`;
  chomp($sha1);
  my $sha2 = `sha256sum $targetdir/dists/$deblabel/core/binary-all/Packages.gz | sed -e 's/ .*//'`;
  chomp($sha2);
  print RELEASE "MD5Sum:\n $md5   $size Packages.gz\n";
  print RELEASE "SHA1:\n $sha1   $size Packages.gz\n";
  print RELEASE "SHA256:\n $sha2   $size Packages.gz\n";
  close RELEASE;

  # create the master release file
  open RELEASE,">$targetdir/dists/$deblabel/Release" or
    die "Cannot write release file for master: $!\n";
  print RELEASE <<EOF;
Archive: $deblabel
Components: core
Origin: EGI IGTF Liaison Function
Label: EGI Trust Anchor Distribution
Date: $debiandate
Suite: $deblabel
Architectures: all @debarchs
EOF
  open FLIST,"cd $targetdir/dists/$deblabel && find . -type f |" or die "Cannot list files: $!\n";
  my ($md5string,$sha1string,$sha256string);
  while (<FLIST>) {
    chomp($_);
    ( my $fname = $_) =~ s/^\.\///;
    my $size = (stat("$targetdir/dists/$deblabel/$fname"))[7];
    my $md5 = `md5sum $targetdir/dists/$deblabel/$fname | sed -e 's/ .*//'`; chomp($md5);
    my $sha1 = `sha1sum $targetdir/dists/$deblabel/$fname | sed -e 's/ .*//'`; chomp($sha1);
    my $sha2 = `sha256sum $targetdir/dists/$deblabel/$fname | sed -e 's/ .*//'`; chomp($sha2);
    if ( $fname =~ /binary-/ ) {
      foreach my $arch ( qw(all i386 amd64 ia64 sparc powerpc kfreebsd-i386 kfreebsd-amd64) ) {
        ( my $archname = $fname ) =~ s/binary-all/binary-$arch/;
        $md5string .= sprintf " %s %10d %s\n",$md5,$size,$archname;
        $sha1string .= sprintf " %s %10d %s\n",$sha1,$size,$archname;
        $sha256string .= sprintf " %s %10d %s\n",$sha2,$size,$archname;
      }
    } else {
      $md5string .= sprintf " %s %10d %s\n",$md5,$size,$fname;
      $sha1string .= sprintf " %s %10d %s\n",$md5,$size,$fname;
      $sha256string .= sprintf " %s %10d %s\n",$md5,$size,$fname;
    }
  }
  print RELEASE "MD5Sum:\n$md5string";
  print RELEASE "SHA1:\n$sha1string";
  print RELEASE "SHA256:\n$sha256string";
  close RELEASE;

  do {
    chomp(my $gpghome=`awk '/%_gpg_path/ { print \$2 }' \$HOME/.rpmmacros`);
    chomp(my $gpgkey=`awk '/%_gpg_name/ { print \$2 }' \$HOME/.rpmmacros`);
    my $cmd="cd $targetdir/dists/$deblabel/ && ".
            "gpg --homedir=$gpghome --default-key=$gpgkey -o Release.gpg -bas Release";
    #print "WARNING WARNING NOT YET Executing GPG signing command:\n  $cmd\n";
    $nosign or system($cmd);

    # now the new InRelease file
    my $cmd="cd $targetdir/dists/$deblabel/ && ".
            "gpg --homedir=$gpghome --default-key=$gpgkey -o InRelease --digest-algo SHA256 -a -s --clearsign Release";
    print "Executing GPG InRelease signing command:\n  $cmd\n";
    $nosign or system($cmd);


  };

  return 1;
}

# ###########################################################################
# supporting routines
#

sub uniq(@) {
  my %hash  = map { $_, 1 } @_;
  return keys %hash;
}

sub parse_csv {
  my $text = shift; ## record containing comma-separated values
  my @new = ();
  push(@new, $+) while $text =~ m{
    ## the first part groups the phrase inside the quotes
    "([^\"\\]*(?:\\.[^\"\\]*)*)",?\s*
      | ([^,]+),?\s*
      | ,\s*
    }gx;
  push(@new, undef) if substr($text, -1,1) eq ',';
  return @new; ## list of values that were comma-spearated
}

# ----------------------------------------------------------------------------
# copyWithExpansion($src,$dst,%table)
#
sub copyWithExpansion($$%) {
  my ($src,$dst,%table) = @_;

  open SRC,"$src" or die "$src: $!";
  open DST,">$dst" or die "$dst: $!";

  while (my $l=<SRC>) {
    foreach my $tok ( keys %table ) {
      defined $table{$tok} or $table{$tok}="";
      $l=~s/\@$tok\@/$table{$tok}/g;
    }
    print DST $l;
  }
  close SRC;
  close DST;
  return 1;
}

sub substituteWithExpansion($%) {
  my ($src,%table) = @_;

  while ( $src =~ m/@\w+@/m ) {
    foreach my $tok ( keys %table ) {
      defined $table{$tok} or $table{$tok}="";
      $src=~s/\@$tok\@/$table{$tok}/gm;
    }
  }
  return $src;
}
# ----------------------------------------------------------------------------
# minimally quote a string using urlencoding
#
sub quoteDN($) {
  my ($s) = @_;
  $s =~ s/\%/\%25/g;
  $s =~ s/\"/\%22/g;
  return "\"$s\"";
}

# ---------------------------------------------------------------------------
# retrieve version from web url (based on IGTF distro)
sub getReleaseVersion($) {
  my ($srcurl) = @_;
  my $ua = LWP::UserAgent->new;
  my $response = $ua->get("$srcurl/version.txt");
  if ( ! $response->is_success ) {
    die "Cannot retrieve version from $srcurl: ".$response->status_line."\n";
  }
  my $vdata = $response->content;
  $vdata =~ m/^This is version (\d+\.\d+)/ or
    die "Cannot find version number in $srcurl/version.txt\n$vdata\n";
  return $1;
}


# ---------------------------------------------------------------------------
# retrieve data content from URL, optionally also writing to local disk
#
sub getURLContent($) {
  my ($url,$file) = @_;

  my $lua = LWP::UserAgent->new;
  my $resp = $lua->get("$url");
  die "Cannot retrieve $url: ".$resp->status_line."\n"
    unless $resp->is_success;

  if ( defined $file ) {
    if ( Scalar::Util::openhandle($file) ) {
      print $file $resp->content; 
    } elsif ( $file ne "" ) {
      open OF,">$file" or die "Cannot open $file to store $url: $!\n";
      print OF $resp->content; 
      close OF;
    } else {
      die "You gave $file to getURLContent, but what the heck is \$file?\n";
    }
  }
  return $resp->content;
}


# ---------------------------------------------------------------------------
# read all CA data, accreditation status, etc, from IGTF
# - get the information on all current CAs (incl. their status) from
#   reading the tar-balls in {accredited,experimental,unaccredited}
sub readCADataFromIGTF($) {
  my ($srcurl) = @_;
  my %cadata;

  foreach my $folder ( qw/ accredited experimental unaccredited /) {
    my $htmllisting = &getURLContent("$srcurl/$folder/tgz/");

    foreach my $line ( split(/[\n\r]+/,$htmllisting) ) {
      $verb>3 and print "Considering html multiview line: $line\n";
      $line =~ m/.*<a href=\"ca_([^\"]+)-$version\.tar.gz\">ca_.*/i or next;
      my $name = ($1);
      $verb>1 and print "Reading CA data for $name\n";

      # we need a local copy of the tar data, since Archive::Tar cannot read
      # from string yes. Other packages can be read in-line
      my ($tmptar, $tmptarname ) = 
        tempfile("/tmp/tgz-ca_$name-$version.XXXXXX", CLEANUP=>1);
      $tmptar or die "Cannot open tempfile for tgz $name: $!\n";
      $cadata{$name}->{"tardata"} = 
        &getURLContent("$srcurl/$folder/tgz/ca_$name-$version.tar.gz",$tmptar);
      close $tmptar;

      # parse info file
      my $tar = Archive::Tar->new("$tmptarname");
      if ( $tar->contains_file("ca_$name-$version/$name.pem") ) {
        $cadata{$name}->{"infodata"} = 
          $tar->get_content("ca_$name-$version/$name.info");
        $cadata{$name}->{"pemdata"} = 
          $tar->get_content("ca_$name-$version/$name.pem");
        $cadata{$name}->{"inforef"} = ConfigTiny->new();
        $cadata{$name}->{"inforef"}->read_string($cadata{$name}->{"infodata"});
        $cadata{$name}->{"RPMdata"} = 
          &getURLContent("$srcurl/$folder/RPMS/".
                         "ca_$name-$version-$uprelease.noarch.rpm");
        $cadata{$name}->{"SRPMdata"} = 
          &getURLContent("$srcurl/$folder/SRPMS/".
                         "ca_$name-$version-$uprelease.src.rpm");
        $cadata{$name}->{"DEBdata"} = 
          &getURLContent("$srcurl/dists/igtf/$folder/binary-all/".
                         "ca-".lc($name)."-$version-$uprelease.deb");
        $verb>3 and print "  ".$cadata{$name}->{"inforef"}->{_}->{subjectdn}.
              "\n  ".$cadata{$name}->{"inforef"}->{_}->{status}."\n";
      }

      unlink($tmptarname);
    }

  }
  $verb>3 and do {
    my $s = Dumper(%cadata); $s =~ s/[^\x0a\x20-\x7F]+/./g;
    print "CADATA read into hash \%cadata:\n$s\n";
  };
  return %cadata;
}

sub readPolicyDataFromIGTF($$@) {
  my ($srcurl,$name,@policies) = @_;
  my %policydata;

  my $tardata = &getURLContent("$srcurl/accredited/tgz/".
                               "ca_policy_$name-$version.tar.gz");
  $policydata{"$name"}->{"tardata"} = $tardata;
  $policydata{"$name"}->{"SRPMdata"} = 
    &getURLContent("$srcurl/accredited/SRPMS/".
                   "ca_policy_$name-$version-$uprelease.src.rpm");

  # since Archive::Tar does not yet read stringified archives, write first
  my ($tmptar, $tmptarname ) = 
    tempfile("/tmp/tgz-ca_$name-$version.XXXXXX", CLEANUP=>1);
  $tmptar or die "Cannot open tempfile for tgz $name: $!\n";
  print $tmptar $tardata;
  close $tmptar;

  # parse info file
  my $tar = Archive::Tar->new("$tmptarname");
  foreach my $pol ( @policies ) {
    if($tar->contains_file("ca_policy_$name-$version/policy-$name-$pol.info")){
      $policydata{"$name-$pol"}->{"infodata"} = 
        $tar->get_content("ca_policy_$name-$version/policy-$name-$pol.info");
      $policydata{"$name-$pol"}->{"inforef"} = ConfigTiny->new();
      $policydata{"$name-$pol"}->{"inforef"}->read_string(
          $policydata{"$name-$pol"}->{"infodata"}
        );
      $verb>1 and print "Read policy $name-$pol (".
                        length($policydata{"$name-$pol"}->{"infodata"}).")\n";
      $pol =~ m/classic|mics|slcs|iota/ and do {
        $policydata{"$name-$pol"}->{"RPMdata"} = 
          &getURLContent("$srcurl/accredited/RPMS/".
                         "ca_policy_$name-$pol-$version-$uprelease.noarch.rpm");
        $policydata{"$name-$pol"}->{"DEBdata"} = 
          &getURLContent("$srcurl/dists/igtf/accredited/binary-all/".
                          "ca-policy-$name-$pol-$version-$uprelease.deb");
      };
      $verb>2 and do {
        my $s=Dumper($policydata{"$name-$pol"}); $s =~ s/[^\x0a\x20-\x7F]+/./g;
        print "Policy $name-$pol read in hash:\n$s\n";
      };
    }
  }
  unlink($tmptarname);

  return %policydata;
}

# ---------------------------------------------------------------------------
# command-line help
sub help(%) {
  my %opt = @_;
  my $length=8+length($0);
  print "Usage: $0";
  foreach my $v ( keys %opt ) {
    if ($length+length($v)+2 > 77 ) { print "\n    "; $length=5; }
    print " [$v]" and $length+=(length($v)+3);
  }
  print <<EOF;

Put together the EGI policy distribution based on the web publication of
the IGTF Distribution version. Write the result to the non-existent or
empty directory <output>.
EOF
  return 1;
}

sub makeLicHeaders($) {
  my ($i) = @_;
  my %s;

  $s{"egi-core"} = <<EOF;
# Copyright (c) Stichting EGI, $year
#
# This work is licensed under the Creative Commons Attribution-Noncommercial
# 3.0 License. To view a copy of this license, visit 
#   http://creativecommons.org/licenses/by-nc/3.0/
# Generated by $0
#
# Warning: this template ONLY encodes the EGI.eu policy on endorsed CAs
# and may NOT have the complete set of IGTF accredited CAs.
# *** Use of this template is AT YOUR OWN RISK ***
EOF

  $s{"lcg"} = <<EOF;
# Copyright (c) FOM Nikhef, National Institute for Subatomic Physics, $year
#           for the world-wide LHC Computing Grid collaboration
#           
# This work is licensed under the Creative Commons Attribution-Noncommercial
# 3.0 License. To view a copy of this license, visit 
#   http://creativecommons.org/licenses/by-nc/3.0/
# Generated by $0
#
# Warning: this template ONLY encodes the wLCG policy on endorsed CAs
# and may contain additional trust anchors
# *** Use of this template is AT YOUR OWN RISK ***
EOF

  return $s{$i} if defined $i;
  return %s;
}

sub makeNRMSFile($$$) {
  my ($fname,$pol,$nrmsrelease) = @_;

  $nrmsrelease = $release unless defined $nrmsrelease;
  my ($major,$minor) = split(/\./,$version);

  my $releasenotes = do { local $/; 
    open my $notes,"<doc/ca-policy-$pol-readme-$version.txt"; 
    <$notes>;
  };
  $releasenotes = &substituteWithExpansion($releasenotes,
      ( "VERSION" => $version, "RELEASE" => $release )
    );

  open my $nrms,">",$fname or die "Cannot open NRMS $fname for write: $!\n";
  print $nrms <<EOF;
<?xml version="1.0" encoding="ISO-8859-1"?>
<release:release xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:noNamespaceSchemaLocation="release.xsd"
    xmlns:release="http://repo.egi.eu/0.2/release"
    xmlns:repository="http://repo.egi.eu/0.2/repository"
    xmlns:repofile="http://repo.egi.eu/0.2/repofile">
    <release:productName>EGI Core Trust Anchor Distribution</release:productName>
    <release:productShortName>cas</release:productShortName>
    <release:technologyProvider>Stichting EGI</release:technologyProvider>
    <release:technologyProviderShortName>EGI-IGTF</release:technologyProviderShortName>
    <release:contact>egi-igtf-liaison\@nikhef.nl</release:contact>
    <release:technicalContact>egi-igtf-liaison\@nikhef.nl</release:technicalContact>
    <release:description>Trust anchors endorsed by the EGI.eu foundation</release:description>  
    <release:documentationLink>https://wiki.egi.eu/wiki/EGI_IGTF_Release</release:documentationLink>
    <release:documentationLink>https://wiki.egi.eu/wiki/EGI_IGTF_Release_Process</release:documentationLink>
    <release:keyword>ca-policy-egi-core trust CA IGTF</release:keyword>
    <release:ISODate>$today</release:ISODate>
    <release:incremental>false</release:incremental>
    <release:emergency>false</release:emergency>
    <release:majorVersion>$major</release:majorVersion>
    <release:minorVersion>$minor</release:minorVersion>
    <release:revisionVersion>$nrmsrelease</release:revisionVersion>
    <release:releaseNotes>$releasenotes</release:releaseNotes>
    <release:changeLog>Update of the EGI Trust Anchors to release $version-$release</release:changeLog>
    <release:synchProtocol>rsync</release:synchProtocol>
    <release:synchURL>rsync://egi-igtf.ndpf.info/egi-igtf/egi/</release:synchURL>  
    <repofile:yumRepofile filename="EGI-trustanchors.repo" relativePath="current/repo-files/">
    <repofile:yumRepository>
    <repofile:id>EGI-trustanchors</repofile:id>
    <repofile:name>EGI-trustanchors</repofile:name>
    <repofile:baseUrl type="relative">current/</repofile:baseUrl>
    <repofile:enabled>1</repofile:enabled>
    <repofile:gpgcheck>1</repofile:gpgcheck>
    <repofile:gpgkey>http://repository.egi.eu/sw/production/cas/1/GPG-KEY-EUGridPMA-RPM-3</repofile:gpgkey>
    </repofile:yumRepository>
    </repofile:yumRepofile>
</release:release>
EOF




}
